//All original code by Agent_Ash (aka Jekyll Grim Payne)
//further edits by BROS_ETT_311

// This array makes sure every player has a
// FollowerControl item in their inventory.
class FollowerSpawningHandler : EventHandler
{	
	override void PlayerEntered(playerEvent e)
	{
		// First, don't do anything if it's a TITLEMAP,
		// or the player is invalid, or doesn't have
		// a PlayerPawn attached for some reason:
		if (level.Mapname == "TITLEMAP")
			return;
		if (!PlayerInGame[e.PlayerNumber])
			return;
		let plr = players[e.PlayerNumber].mo;
		if (!plr)
			return;
		// Check if the player has the control item
		// in their inventory:
		let cont = FollowerControl(plr.FindInventory("FollowerControl"));
		// If they do, respawn their followers:
		if (cont)
			cont.RespawnFollowers();
		// If they don't, give them the item:
		else
			plr.GiveInventory("FollowerControl", 1);		
	}
	
	/*override void WorldThingSpawned (worldevent e) 
    { 
		static const name followerlist[] =
		{
			"Marine_Rifle",
			"Marine_Shotgun",
			"Marine_Minigun",
			"Marine_Rocket",
			"Marine_PlasmaGun",
			"Marine_BFG"
		};
		
		class<Actor> who = followerlist[(random(0, followerlist.Size() - 1))];
        //check if actor exists and is on the list:
        if (e.thing && e.thing is "who")
            FollowerControl.CreateFollower(activator, who);
    }*/
}

// This is a data container that contains the data
// for the follower. The 'mo' and 'type' fields are
// mandatory. More fields can be added, if necessary;
// in that case they will need to be added to the 
// SpawnFollower() function defined in FollowerControl
// (see further below).
class FollowerData play
{
	Actor mo;
	class<Actor> type;
	int health;
}

// This item must be placed in the player's inventory,
// which is done via FollowerSpawningHandler.
// This item stores an array of FollowerData instances,
// thus saving all follower-related data and transferring
// it between maps (since items don't get stripped).
class FollowerControl : Inventory
{
	array <FollowerData> followers;
	
	Default
	{
		+INVENTORY.UNDROPPABLE
		+INVENTORY.UNTOSSABLE
		Inventory.maxamount 1;
	}
	
	// This is the function necessary to initially create
	// a follower that didn't exist before.
	// The first argument must be a pointer to the PlayerPawn,
	// the second--the name of the follower class:
	static void CreateFollower(Actor ppawn, class<Actor> follower)
	{
		if (!ppawn || !ppawn.player)
			return;
		// Find the control item in player's inventory:
		let cont = FollowerControl(ppawn.FindInventory("FollowerControl"));
		if (!cont)
			return;
		// Create a new instance of FollowerData:
		let fdata = New("FollowerData");
		if (!fdata)
		{
			console.printf("Error: couldn't create an instance of FollowerData");
			return;
		}
		// Spawn a follower:
		cont.SpawnFollower(fdata, follower, ppawn.pos);
		// Record the related instance of FollowerData
		// in the array in FollowerControl:
		cont.followers.Push(fdata);
	}
	
	// This is a base follower-spawning function.
	// It's not meant to be used directly; instead
	// it's used by CreateFollower() and RespawnFollowers()
	// to spawn and record a follower when needed.
	void SpawnFollower(FollowerData fdata, class<Actor> follower, vector3 pos)
	{
		if (!fdata)
		{
			console.printf("Error: invalid FollowerData reference passed. Spawning follower canceleld");
			return;
		}
		// Spawn the follower:
		let flwr = Spawn(follower, pos);
		if (!flwr)
		{
			console.printf("Error: couldn't spawn %s at %.1f.%.1f.%.1f", follower.GetClassName(), pos.x, pos.y, pos.z);
			return;
		}		
		flwr.bFRIENDLY = true; //make it friendly
		fdata.mo = flwr; //save a pointer to it in FollowerData
		fdata.type = follower; //record its class type
		
		// This is a debug function. Uncomment to get info at follower spawn:
		
		//console.printf("Spawned %s at at %.1f.%.1f.%.1f with %d health", follower.GetClassName(), pos.x, pos.y, pos.z, fdata.health);
	}		
	
	// This function reads the FollowerData array
	// and respawns followers based on the 'type'
	// field. It's meant to be called by the 
	// event handler:
	void RespawnFollowers()
	{
		if (!owner)
			return;
		// Iterater through the array:
		for (int i = 0; i < followers.Size(); i++)
		{
			if (!followers[i])
				continue;
			// Get the 'type' (class name) of the 
			// follower we need to spawn:
			let type = followers[i].type;
			if (!type)
			{
				console.printf("Error: index %d of the followers array has no type", i);
				followers.Delete(i);
				continue;
			}
			// Spawn the follower and record it again:
			SpawnFollower(followers[i], type, owner.pos);
		}
	}
	
	// This is to improve performance: this control
	// item can't exist in the world, so it doesn't
	// need a Tick():
	override void Tick() {}
	
	// This is called every tic while the item is
	// in the player's inventory to make sure
	// FollowerData tracks every follower's health:
	override void DoEffect()
	{
		super.DoEffect();
		
		if (!owner || !owner.player)
			return;
		
		for (int i = 0; i < followers.Size(); i++)
		{
			if (!followers[i])
				continue;
			let mo = followers[i].mo;
			if (!mo)
				continue;
			// If the follower is dead, remove its data
			// from the array:
			if (mo.health <= 0)
			{
				followers.Delete(i);
				continue;
			}
			followers[i].health = mo.health;
		}
	}
}


// This is an example basic follower spawner. 
// It spawns and record a follower once you press 
// Use in front of it:
class FollowerSpawner : Actor// Replaces Blursphere
{
    //static const Class<Actor> followerlist[] =
	// use 'name' instead of 'class<Actor>':
    static const name followerlist[] =
    {
        "Marine_Rifle",
        "Marine_Shotgun",
        "Marine_Minigun",
        "Marine_Rocket",
        "Marine_PlasmaGun",
        "Marine_BFG"
    };
    
    Default
    {
		/*+SHOOTABLE
		+FLOORCLIP
		+FRIENDLY
		-COUNTKILL
		+FLOORCLIP*/
        +SOLID
        +USESPECIAL
        Activation THINGSPEC_Activate;
        /*Renderstyle "Translucent";
		Radius 24;
		Height 48;
		Speed 0;
		Health 300;
		Mass 999999;
		Scale 1.0;
		Decal "BloodSuper";
		Painchance 255;
		DamageType "Taunt";
		damagefactor "Trample", 0.0;
		damagefactor "Cut", 10.0;
		damagefactor "Taunt", 0.0;
		damagefactor "headcut", 0.0;
		damagefactor "KillMe", 0.0;
		damagefactor "Fatality", 99.0;
		damagefactor "Melee", 99.0;
		damagefactor "Kick", 99.0;
		damagefactor "GibRemoving", 0.0;
		damagefactor "TeleportRemover", 0.0;
		DamageFactor "CauseObjectsToSplash", 0.0;
		DeathSound "*death";
		PainSound "*pain50";
		BloodType "Brutal_Blood";
		DropItem "PoleWithNothing";*/
    }
    
	override void Activate(actor activator)
    {
        // Check that activator is valid and is a player:
        if (!activator || !activator.player)
            return;
        // Choose an entry from the followerlist array randomly
        // and cast it as a class:
		//let who = followerlist[(random(0, followerlist.Size() - 1))];        
        class<Actor> who = followerlist[(random(0, followerlist.Size() - 1))];
        // If the chosen class exists, spawn it:
        if (who)
        {
            // Spawn an Imp follower at the player's position:
            FollowerControl.CreateFollower(activator, who);
            //SetStateLabel("Death"); //the spawner disappears afterwards
			SetStateLabel("SetFree"); //the spawner disappears afterwards
        }
    }
	/*
    States
    {
    Spawn:
	    TNT1 A 0 A_Stop;
        TNT1 A 0 A_CheckSight("Spawn2");
		CAPT A 1 A_Stop;
        CAPT ABCB 15 A_Stop;
		TNT1 A 0 A_CustomMissile ("OrderTitle3", 50, 0, 0, 2, 90);
        TNT1 A 0 A_SpawnItem ("KillMe");
        Loop;

    Spawn2:
        CAPT A 5;
        Goto Spawn;

    SetFree:
    Death.Cut:
    Death.Melee:
	Death.SuperPunch:
    Death.Fatality:
    Death.Kick:
	Active:
		TNT1 A 0 A_Playsound ("TAUNT");
		TNT1 A 0 A_ChangeFLag ("NOPAIN", 1);
        CAPT F 12;
        TNT1 A 0 A_NoBlocking;
		//TNT1 A 0 A_SpawnItemEx ("RandomMarine",0,0,0,0,0,0,0,SXF_NOCHECKPOSITION,0);
        CAPT N 1;
		TNT1 A 0 A_ChangeFLag ("SHOOTABLE", 0);
		TNT1 A 0 A_ChangeFLag ("SOLID", 0);
        CAPT N 0;
        Stop;
	Pain:
	Pain.Blast:
	Pain.ExplosiveImpact:
	Pain.Explosive:
	Pain.Bullet:
	Pain.SSG:
	Pain.CutLess:
	Pain.ImpFatalityMarine:
	Pain.Rip:
        CAPT P 1;
		TNT1 A 0 A_Stop;
        CAPT P 5 A_Pain;
		TNT1 A 0 A_Stop;
        Goto Spawn;
	Death:
	Death.Bullet:
		TNT1 A 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
		PLAY H 10 A_PlayerScream;
		
		PLAY I 10 A_NoBlocking;
		PLAY J 10 ;
		TNT1 A 0 A_SpawnItem("RemoveMarine");
		PLAY KLM 10;
		TNT1 A 0 A_SpawnItem ("MediumBloodSpot");
		PLAY N -1;
		Stop;

    Death.Eat:
        TNT1 A 0 A_GiveToTarget("EatMe",1);
		TNT1 A 0 A_CustomMissile ("XDeath1", 1, 0, random (0, 360), 2, random (0, 160));
		TNT1 A 0 A_CustomMissile ("XDeath4", 1, 0, random (0, 360), 2, random (0, 160));
		TNT1 A 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL1 A 10 A_Playsound("BPSLOP", CHAN_AUTO);
        XPL1 B 20 A_NoBlocking;
		TNT1 A 0 A_SpawnItem("RemoveMarine");
        XPL1 CDE 10 ;
		TNT1 A 0 A_SpawnItem ("MediumBloodSpot");
        XPL1 E -1;
        Stop;

    Death.Cut:
	Death.Saw:
		TNT1 A 0 A_CustomMissile ("XDeath1", 1, 0, random (0, 360), 2, random (0, 160));
		TNT1 A 0 A_CustomMissile ("XDeath2", 1, 0, random (0, 360), 2, random (0, 160));
		TNT1 A 0 A_CustomMissile ("XDeath3", 1, 0, random (0, 360), 2, random (0, 160));
		TNT1 A 0 A_CustomMissile ("XDeath4", 1, 0, random (0, 360), 2, random (0, 160));
        TNT1 A 0 A_CustomMissile ("RipGuts", 50, 0, random (0, 360), 2, random (0, 160));
        TNT1 A 0 A_CustomMissile ("XDeathHalfMarine", 50, 0, random (0, 360), 2, random (0, 160));
        TNT1 AA 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL2 A 10 A_Playsound("BPSLOP", CHAN_AUTO);
        TNT1 AA 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL2 B 20 A_NoBlocking;
		TNT1 A 0 A_SpawnItem("RemoveMarine");
        TNT1 AA 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL2 C 10 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        TNT1 AA 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL2 DE 10 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
		TNT1 A 0 A_SpawnItem ("MediumBloodSpot");
        XPL2 E -1;
        Stop;

    Death.Slime:
        TNT1 A 0 A_PlaySound("BIGSCREA");
		TNT1 A 0 A_SpawnItem("RemoveMarine");
        PMET AABBCCDDEEFFGGHHIIIII 10 A_CustomMissile ("PlasmaSmoke", 25, 0, random (0, 180), 2, random (0, 180));
        PMET IIIIIIIIIIJJJJJJJJJJJJJJJJJJJJJJKKKKKKKKKKKKKKKKKKKKKKKKKKLLLLLLLLLLLLLLL 3 A_CustomMissile ("PlasmaSmoke", 25, 0, random (0, 180), 2, random (0, 180));
        TNT1 A 0 A_NoBlocking;
        TNT1 A -1;
        Stop;

    Death.CutLess:
        TNT1 A 0 A_Jump (128, 3);
        Goto Death;
        TNT1 AAA 0;
		TNT1 A 0 A_CustomMissile ("XDeath1", 1, 0, random (0, 360), 2, random (0, 160));
		TNT1 A 0 A_CustomMissile ("XDeath2", 1, 0, random (0, 360), 2, random (0, 160));
		TNT1 A 0 A_CustomMissile ("XDeath3", 1, 0, random (0, 360), 2, random (0, 160));
		TNT1 A 0 A_CustomMissile ("XDeath4", 1, 0, random (0, 360), 2, random (0, 160));
        TNT1 A 0 A_CustomMissile ("RipGuts", 50, 0, random (0, 360), 2, random (0, 160));
        TNT1 A 0 A_CustomMissile ("XDeathHalfMarine", 50, 0, random (0, 360), 2, random (0, 160));
        TNT1 AA 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL2 A 10 A_Playsound("BPSLOP", CHAN_AUTO);
		TNT1 A 0 A_SpawnItem("RemoveMarine");
        TNT1 AA 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL2 B 20 A_NoBlocking;
        TNT1 AA 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL2 C 10 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        TNT1 AA 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL2 DE 10 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
		TNT1 A 0 A_SpawnItem ("MediumBloodSpot");
        XPL2 E -1;
        Stop;

    Death.Rip:
        TNT1 AAAAAAAAAAA 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL3 A 10 A_Playsound("BPSLOP", CHAN_AUTO);
        TNT1 AAAAA 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL3 B 20 A_NoBlocking;
		TNT1 A 0 A_SpawnItem("RemoveMarine");
        TNT1 A 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL3 C 10 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        TNT1 A 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL3 D 10 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        TNT1 A 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL3 E 10 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        TNT1 A 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL3 F 10 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        TNT1 A 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL3 F 10 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
		TNT1 A 0 A_SpawnItem ("MediumBloodSpot");
        XPL3 F -1;
        Stop;
	Death.Massacre:
	Death.Explosives:
	XDeath:
		TNT1 A 0 ThrustThingZ(0,60,0,1);
        TNT1 A 0 A_CustomMissile ("MarineXDeath", 0, 0, random (0, 360), 2, random (0, 160));
		PPOD A 0 A_SpawnItemEx("BasicMarineGib1", 0, 0, 0, 0, 0, 0, 0, SXF_TRANSFERTRANSLATION);
		PPOD A 0 A_SpawnItemEx("BasicMarineGib2", 0, 0, 0, 0, 0, 0, 0, SXF_TRANSFERTRANSLATION);
		PPOD A 0 A_SpawnItemEx("BasicMarineGib3", 0, 0, 0, 0, 0, 0, 0, SXF_TRANSFERTRANSLATION);
		PPOD A 0 A_SpawnItemEx("BasicMarineGib4", 0, 0, 0, 0, 0, 0, 0, SXF_TRANSFERTRANSLATION);
        MHEA A 7 A_Playsound("BPSLOP", CHAN_AUTO);
        MHEA B 7 A_NoBlocking;
		TNT1 A 0 A_SpawnItem("RemoveMarine");
        MHEA CD 7;
        MHEA E -1;
		Stop;
		
	 Crush:
	 Death.Stomp:
	    TNT1 AAAAAAAAAAAAAAAAAAAAAAAAA 0 A_CustomMissile ("Brutal_FlyingBloodTrail8", 0, 0, random (0, 360), 2, random (0, 360));
		TNT1 AAAAAA 0 bright A_CustomMissile ("SuperGoreSpawner", 5, 0, random (0, 360), 2, random (30, 180));
		TNT1 AAAAAA 0 bright A_CustomMissile ("XDeath1", 5, 0, random (0, 360), 2, random (30, 180));
		TNT1 AA 0 bright A_CustomMissile ("XDeath2", 55, 0, random (0, 360), 2, random (30, 180));
		TNT1 AA 0 bright A_CustomMissile ("XDeath3", 55, 0, random (0, 360), 2, random (30, 180));
		TNT1 A 0 A_SpawnItem ("GrowingBloodPool");
		TNT1 A 0 A_SpawnItem ("CrushedRemains");
		TNT1 A 1;
		TNT1 A 1 A_Playsound("BPSLOP", CHAN_AUTO);
		TNT1 A 1 A_NoBlocking;
		Stop;

	Death.Blast:
	Death.SuperPunch:
	Death.SSG:
	    PLAY O 2 A_FaceTarget;
		TNT1 AAAA 0 A_CustomMissile ("XDeath1", 32, 0, random (170, 190), 2, random (0, 40));
		TNT1 AAAA 0 A_CustomMissile ("XDeath2", 32, 0, random (170, 190), 2, random (0, 40));
		TNT1 AAAA 0 A_CustomMissile ("XDeath3", 32, 0, random (170, 190), 2, random (0, 40));
		TNT1 AAAA 0 A_CustomMissile ("Brutal_FlyingBlood", 32, 0, random (170, 190), 2, random (0, 40));
		TNT1 AAAA 0 A_CustomMissile ("Brutal_FlyingBloodFaster", 32, 0, random (170, 190), 2, random (0, 40));
		TNT1 AAAA 0 A_CustomMissile ("Brutal_FlyingBloodMuchFaster", 32, 0, random (170, 190), 2, random (0, 40));
		TNT1 AAAA 0 A_CustomMissile ("SuperWallRedBlood", 32, 0, random (170, 190), 2, random (0, 40));
		TNT1 AAAA 0 A_CustomMissile ("SuperWallRedBlood", 46, 0, random (170, 190), 2, random (0, 40));
		TNT1 A 0 A_CustomMissile ("XDeathArm1", 32, 0, random (170, 190), 2, random (0, 40));
        TNT1 A 0 A_Playsound("BPSLOP", CHAN_AUTO);
		TNT1 A 0 A_NoBlocking;
		TNT1 A 0 A_SpawnItem("RemoveMarine");
        PLAY OPQRSTU 8;
		TNT1 A 0 A_SpawnItem ("MediumBloodSpot");
        PLAY U -1;
        Stop;

    Death.Rape:
		TNT1 AAAAAAAAAAAAAA 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL4 A 20 A_Playsound("BPSLOP", CHAN_AUTO);
		TNT1 A 0 A_CustomMissile ("XDeath1", 1, 0, random (0, 360), 2, random (0, 160));
		TNT1 A 0 A_CustomMissile ("XDeath2", 1, 0, random (0, 360), 2, random (0, 160));
		TNT1 A 0 A_CustomMissile ("XDeath3", 1, 0, random (0, 360), 2, random (0, 160));
		TNT1 A 0 A_CustomMissile ("XDeath4", 1, 0, random (0, 360), 2, random (0, 160));
        TNT1 AAAA 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL4 B 20 A_NoBlocking;
		TNT1 A 0 A_SpawnItem("RemoveMarine");
        TNT1 AAA 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL4 C 20;
        TNT1 AAA 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL4 D 20;
        TNT1 AAAA 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL4 E 20;
        TNT1 AAA 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
        XPL4 F 20;
        TNT1 AAAA 0 A_CustomMissile ("MuchBlood", 50, 0, random (0, 360), 2, random (0, 160));
		TNT1 A 0 A_SpawnItem ("MediumBloodSpot");
        XPL4 F -1;
        Stop;

    Death.plasma:
        TNT1 A 0;
        TNT1 A 0 A_Playsound("BPSLOP", CHAN_AUTO);
        TNT1 A 0 A_NoBlocking;
		TNT1 A 0 A_SpawnItem("RemoveMarine");
        TNT1 AAA 0 A_CustomMissile ("Brutal_FlyingBlood", 50, 0, random (0, 360), 2, random (0, 160));
	    TNT1 A 0 A_CustomMissile ("XDeathBurnedArm", 32, 0, random (0, 360), 2, random (0, 160));
	    TNT1 A 0 A_CustomMissile ("XDeathBurnedLeg", 32, 0, random (0, 360), 2, random (0, 160));
	    TNT1 A 0 A_CustomMissile ("XDeathBurnedSkull", 32, 0, random (0, 360), 2, random (0, 160));
	    TNT1 A 0 A_CustomMissile ("XDeathBurnedMeat", 32, 0, random (0, 360), 2, random (0, 160));
	    TNT1 A 0 A_CustomMissile ("XDeathBurnedMeat2", 32, 0, random (0, 360), 2, random (0, 160));
	    TNT1 A 0 A_CustomMissile ("XDeathBurnedMeat3", 32, 0, random (0, 360), 2, random (0, 160));
		EXPL AAAAAA 0 A_CustomMissile ("ExplosionSmoke", 32, 0, random (0, 360), 2, random (0, 360));
        TNT1 A 1;
        TNT1 A -1;
        Stop;


	Death.GreenFire:
        TNT1 A 0;
        TNT1 A 0 A_Playsound("BPSLOP", CHAN_AUTO);
        TNT1 A 0 A_NoBlocking;
		TNT1 A 0 A_SpawnItem("RemoveMarine");
        TNT1 AAAA 0 A_CustomMissile ("Brutal_Blood", 30, 0, random (0, 360), 2, random (0, 160));
		
	    TNT1 A 0 A_CustomMissile ("XDeathBurnedArm", 32, 0, random (0, 360), 2, random (0, 160));
	    TNT1 A 0 A_CustomMissile ("XDeathBurnedLeg", 32, 0, random (0, 360), 2, random (0, 160));
	    TNT1 A 0 A_CustomMissile ("XDeathBurnedSkull", 32, 0, random (0, 360), 2, random (0, 160));
	    TNT1 A 0 A_CustomMissile ("XDeathBurnedMeat", 32, 0, random (0, 360), 2, random (0, 160));
	    TNT1 A 0 A_CustomMissile ("XDeathBurnedMeat2", 32, 0, random (0, 360), 2, random (0, 160));
	    TNT1 A 0 A_CustomMissile ("XDeathBurnedMeat3", 32, 0, random (0, 360), 2, random (0, 160));
		
		EXPL AAAAAAAAAAAAAAAAAAAAAAAA 0 A_CustomMissile ("GreenFlameTrails", 50, 0, random (0, 360), 2, random (0, 360));
		XBRN AAAA 2 BRIGHT A_SpawnItem("GreenFlare",0,43);
        Stop;

    Death.burn:
		PBUR A 1;
		PBUR A 1 A_Scream;
		PBUR A 1 A_NoBlocking;
		TNT1 A 0 A_SpawnItem("RemoveMarine");
		PBUR AAAABBBBCCCC 2 A_CustomMissile ("SmallFlameTrails", 32, 0, random (0, 180), 2, random (0, 180));
	    PBUR DDDDEEEE 2 A_CustomMissile ("SmallFlameTrails", 16, 0, random (0, 180), 2, random (0, 180));
        PBUR EEEEE 4 A_CustomMissile ("SmallFlameTrails", 8, 0, random (0, 180), 2, random (0, 180));
		PBUR EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE 6 A_CustomMissile ("PlasmaSmoke", 8, 0, random (0, 180), 2, random (0, 180));
        PBUR E -1;
        Stop;

    Death.Fire:
	Death.Flames:
	Death.Burn:
        TNT1 A 0 A_PlaySound("BIGSCREA");
		BURN W 6 A_Scream;
		BURN X 6 A_NoBlocking;
		TNT1 A 0 A_SpawnItem("RemoveMarine");
		BUR2 ABCD 6 BRIGHT A_CustomMissile ("PlasmaSmoke", 8, 0, random (0, 180), 2, random (0, 180));
		BURN FGHIJKL 6 BRIGHT A_CustomMissile ("PlasmaSmoke", 8, 0, random (0, 180), 2, random (0, 180));
		BURN MNOPQRSTUV 6 BRIGHT A_CustomMissile ("PlasmaSmoke", 8, 0, random (0, 180), 2, random (0, 180));
        BURN VVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVV 6 A_CustomMissile ("PlasmaSmoke", 8, 0, random (0, 180), 2, random (0, 180));
        BURN V -1;
		Stop;
	  
	  
	  Death.ExplosiveImpact:
		NULL A 0 ThrustThingZ(0,30,0,1);
		NULL AAAA 0 A_CustomMissile ("Brains1", 50, 0, random (0, 360), 2, random (0, 160));
		NULL A 0 A_CustomMissile ("Brains2", 50, 0, random (0, 360), 2, random (0, 160));
		NULL A 0 A_CustomMissile ("Brains3", 50, 0, random (0, 360), 2, random (0, 160));
		NULL A 0 A_CustomMissile ("Brains4", 50, 0, random (0, 360), 2, random (0, 160));
		NULL A 0 A_CustomMissile ("Brains5", 50, 0, random (0, 360), 2, random (0, 160));
		NULL AAAA 0 A_CustomMissile ("SmallBrainPiece", random (45, 55), random (5, -5), random (170, 190), 2, random (0, 40));
		NULL AAAA 0 A_CustomMissile ("SmallBrainPiece", 50, 0, random (0, 360), 2, random (0, 160));
		NULL AA 0 A_CustomMissile ("SuperWallRedBlood", 46, 0, random (170, 190), 2, random (0, 40));
        NULL AAAAAAAAAAA 0 A_CustomMissile ("BrainBlood", 50, 0, random (0, 360), 2, random (0, 160));
	    NULL A 0 A_CustomMissile ("XDeathArm1", 35, 0, random (0, 360), 2, random (0, 160));
	    NULL A 0 A_CustomMissile ("XDeath3", 40, 0, random (0, 360), 2, random (0, 160));
	    PPOD A 0 A_SpawnItemEx("BasicMarineGib1", 0, 0, 0, 0, 0, 0, 0, SXF_TRANSFERTRANSLATION);
        NULL AAAA 0 A_CustomMissile ("XDeath1", 40, 0, random (0, 360), 2, random (0, 160));
		NULL A 0 A_Playsound("BPSLOP", CHAN_AUTO);
        XPL6 A 5;
		NULL A 0 A_NoBlocking;
        XPL6 BCDE 5;
        XPL6 F -1;
        Stop;

    Crush:
        TNT1 A 0 A_PlaySound("misc/xdeath4");
        CRS1 A 5;
        CRS1 A -1;
        Stop;

    Death.HKFT:
	Death.BHFT:
	Death.RVFT:
		TNT1 A 1 A_PlayerScream;
		TNT1 A 1 A_NoBlocking;
		TNT1 A 0 A_SpawnItem("RemoveMarine");
        TNT1 A 0 A_GiveToTarget("Curbstomp_Marine",1);
		TNT1 A -1;
        Stop;

    Death.FatalityMarine:
		TNT1 A 1 A_PlayerScream;
		TNT1 A 0 A_GiveToTarget("GoFatality", 1);
		TNT1 A 1 A_NoBlocking;
		TNT1 A 0 A_SpawnItem("RemoveMarine");
        TNT1 A 0 A_GiveToTarget("Fatality_Marine",1);
		TNT1 A -1;
        Stop;
	}*/
}
